;; Improved next-line command.
;; Copyright (C) 1991, 1992 Free Software Foundation, Inc.
;;
;; This file is directly derived from part of GNU Emacs.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 1, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;; Created by: Joe Wells, jbw@bigbird.bu.edu
;; Created on: Sat Feb 29 03:54:09 1992
;; Last modified by: Joe Wells, jbw@bigbird.bu.edu
;; Last modified on: Sat Feb 29 04:11:46 1992
;; Filename: next-line-fix.el
;; Purpose: improved next-line to answer everyone's prayers

;; LCD Archive Entry:
;; next-line-fix|Joe Wells|jbw@cs.bu.edu
;; |improved next-line to answer everyone's prayers
;; |sometime in 1991|1.1|???

;; Advantages:
;;
;; 1. Doesn't add lines at end of buffer unless you want it to.
;;
;; 2. Can be configured with a maximum number of lines to add at the end
;; of the buffer.  A value of 2 for next-line-max-inserted-newlines will
;; result in at most one blank line at the end of the buffer.
;; 
;; 3. If you tell it never to add newlines, then it will move to the end
;; of the buffer instead.  You can easily get back to the column you
;; started in because that is always saved in temporary-goal-column.

(defvar next-line-max-inserted-newlines 2
  "*Maximum number of newlines next-line will create at end of buffer.")

(defun next-line (arg)
  "Move cursor vertically down ARG lines.
If there is no character in the target line exactly under the current column,
the cursor is positioned after the character in that line which spans this
column, or at the end of the line if it is not long enough.

If at the end of the buffer, it will add up to next-line-max-inserted-newlines 
newline characters to allow moving to the next line.

The command \\[set-goal-column] can be used to create
a semipermanent goal column to which this command always moves.
Then it does not try to move vertically.

If you are thinking of using this in a Lisp program, consider
using `forward-line' instead.  It is usually easier to use
and more reliable (no dependence on goal column, etc.)."
  (interactive "p")
  (if (= arg 1)
      (let ((opoint (point)))
	(forward-line 1)
	(if (or (= opoint (point))
		(not (eq (preceding-char) ?\n)))
	    (if (< (- (point)
		      (save-excursion
			(skip-chars-backward "\n")
			(point)))
		   next-line-max-inserted-newlines)
		(insert ?\n))
	  (goto-char opoint)
	  (next-line-internal arg)))
    (next-line-internal arg))
  nil)

;; Compatibility with many Emacs versions.
(or (fboundp 'next-line-internal)
    (fset 'next-line-internal
	  (symbol-function 'line-move)))

(provide 'next-line-fix)
